- x
A data frame of model's parameters, as returned by various functions
of the easystats-packages. May also be a result from
broom::tidy().
- pretty_names
Return "pretty" (i.e. more human readable) parameter
names.
- stars
Add significance stars (e.g., p < .001***).
- digits, ci_digits, p_digits, rope_digits, ic_digits
Number of digits for
rounding or significant figures. May also be "signif" to return significant
figures or "scientific" to return scientific notation. Control the
number of digits by adding the value as suffix, e.g. digits = "scientific4"
to have scientific notation with 4 decimal places, or digits = "signif5"
for 5 significant figures (see also signif()).
- ci_width
Minimum width of the returned string for confidence
intervals. If not NULL and width is larger than the string's length,
leading whitespaces are added to the string. If width="auto", width
will be set to the length of the longest string.
- ci_brackets
Logical, if TRUE (default), CI-values are
encompassed in square brackets (else in parentheses).
- zap_small
Logical, if TRUE, small values are rounded after
digits decimal places. If FALSE, values with more decimal
places than digits are printed in scientific notation.
- preserve_attributes
Logical, if TRUE, preserves all attributes
from the input data frame.
- exact
Formatting for Bayes factor columns, in case the provided data
frame contains such a column (i.e. columns named "BF" or "log_BF").
For exact = TRUE, very large or very small values are then either reported
with a scientific format (e.g., 4.24e5), else as truncated values (as "> 1000"
and "< 1/1000").
- verbose
Toggle messages and warnings.
- ...
Arguments passed to or from other methods.